unit IngPopupButton;

interface

uses
  Windows, SysUtils, Classes, Controls, Buttons, Graphics{, Menus};

type
  TIngPopupButton = class(TSpeedButton)
  private
    fArrowWidth: Integer;
    //fPopupMenu: TPopupMenu;
    procedure SetArrowWidth(const Value: Integer);
    //procedure SetPopupMenu(Value: TPopupMenu);
    procedure UpdateTracking;
  protected
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Align;
    property ArrowWidth: Integer read fArrowWidth write SetArrowWidth default 13;
    property OnMouseDown;
    property OnMouseUp;
    //property PopupMenu: TPopupMenu read fPopupMenu write SetPopupMenu;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Standard', [TIngPopupButton]);
end;

{ TIngPopupButton }

constructor TIngPopupButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  SetBounds(0, 0, 75, 25);
  ShowHint := True;

  fArrowWidth := 13;
end;

procedure TIngPopupButton.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  Pnt: TPoint;
begin
  inherited MouseDown(Button, Shift, X, Y);

  if not Enabled then
    Exit;

  if Button = mbLeft then
  begin
    if not Down then
      FState := bsDown else
      FState := bsExclusive;
    Repaint; // Invalidate;
  end;

  if Assigned(PopupMenu) then
  begin
    Pnt := ClientToScreen(Point(0, Height));
    PopupMenu.Popup(Pnt.X, Pnt.Y);
  end;

  MouseUp(Button, Shift, X, Y);
end;

procedure TIngPopupButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);

  if not Enabled then
  begin
    FState := bsUp;
    Repaint;
  end;

  if GroupIndex = 0 then
  begin
    // Redraw face in case mouse is captured
    FState := bsUp;

    if not (FState in [bsExclusive, bsDown]) then
      Invalidate;
  end else
    begin
      Down := not Down;
      if Down then
        Repaint;
    end;
  Click;
  UpdateTracking;
  Repaint;
end;

procedure TIngPopupButton.Paint;
var
  ARect: TRect;
  X, Y: Integer;
begin
  inherited;

  Canvas.Brush.Color := clBtnFace;
  Canvas.Brush.Style := bsSolid;
  ARect := Rect(Width - fArrowWidth, 2, Width - 2, Height - 2);
  Canvas.FillRect(ARect);

  {Canvas.Pen.Color := clBtnShadow;
  Canvas.MoveTo(ARect.Left + 1, ARect.Top);
  Canvas.LineTo(ARect.Left + 1, ARect.Bottom);}

  with ARect do
  begin
    X := Right - Left;
    X := X div 2;
    Y := Bottom - Top;
    Y := Y div 2;
    Bottom := Bottom - (Y + X div 2) + 1;
    Top := Top + (Y + X div 2) + 1;
    Left := Left + (X div 2);
    Right := (Right - X div 2);
  end;

  if Enabled then
    Canvas.Pen.Color := clBlack else
    Canvas.Pen.Color := clBtnShadow;

  while ARect.Left < ARect.Right + 1 do
  begin
    Canvas.MoveTo(ARect.Left, ARect.Bottom);
    Canvas.LineTo(ARect.Right, ARect.Bottom);
    InflateRect(ARect, -1, 1);
  end;
end;

procedure TIngPopupButton.SetArrowWidth(const Value: Integer);
begin
  if fArrowWidth <> Value then
  begin
    fArrowWidth := Value;
    Repaint;
  end;
end;

{procedure TIngPopupButton.SetPopupMenu(Value: TPopupMenu);
begin
  fPopupMenu := Value;
  if Value <> nil then
  begin
    Value.ParentBiDiModeChanged(Self);
    Value.FreeNotification(Self);
  end;
end;}

procedure TIngPopupButton.UpdateTracking;
begin
  inherited;
  if Flat then
    if Enabled then
      Perform(CM_MOUSELEAVE, 0, 0);
end;

end.
